/* Copyright (c) 1992 AT&T - All rights reserved. */
/* Changes 2000, Scott Schwartz */

#include	<u.h>
#include	<libc.h>

void
exits(char *message)
{

        exit(message != 0);
}       
        
void
_exits(char *message)
{       
        _exit(message != 0);
}

int errstr(char *buf)
{
	extern int errno;

	strncpy(buf, strerror(errno), ERRLEN);
	return 1;
}

int create(char *name, int omode, int perm)
{
        int mode; 
        int fd;

        if (omode & OWRITE) mode = O_WRONLY;
        else if (omode & OREAD) mode = O_RDONLY;
        else mode = O_RDWR;

        if ((fd = open(name, mode|O_CREAT|O_TRUNC, perm)) < 0)
                return fd;

        if (omode & OCEXEC)
                fcntl(fd, F_SETFD, fcntl(fd,F_GETFD,0) | FD_CLOEXEC);
        
        /* XXX - not exactly right, but hopefully good enough. */
        if (omode & ORCLOSE)
                remove(name);

        return fd;
}

char*
getuser(void)
{
	struct passwd *p;

	static char *user = 0;

	if (!user) {
		p = getpwuid(getuid());
		if (p && p->pw_name) {
			user = malloc(strlen(p->pw_name)+1);
			if (user)
				strcpy(user, p->pw_name);
		}
	}
	if(!user)
		user = "unknown";
	return user;
}

#ifdef NEEDSTRERROR
char *
strerror(int n)
{
	extern char *sys_errlist[];
	return sys_errlist[n];
}
#endif /* NEEDSTRERROR */

#ifdef NEEDMEMMOVE
/*
 * memcpy is probably fast, but may not work with overlap
 */
void*
memmove(void *a1, const void *a2, size_t n)
{
	char *s1;
	const char *s2;

	s1 = a1;
	s2 = a2;
	if(s1 > s2)
		goto back;
	if(s1 + n <= s2)
		return memcpy(a1, a2, n);
	while(n > 0) {
		*s1++ = *s2++;
		n--;
	}
	return a1;

back:
	s2 += n;
	if(s2 <= s1)
		return memcpy(a1, a2, n);
	s1 += n;
	while(n > 0) {
		*--s1 = *--s2;
		n--;
	}
	return a1;
}
#endif /* NEEDMEMMOVE */
